#!/usr/bin/env node
// Source: https://github.com/evanw/esbuild/issues/1921#issuecomment-1898197331
import { createRequire } from 'node:module';
import url from 'node:url';
globalThis.require = createRequire(import.meta.url);
globalThis.__filename = url.fileURLToPath(import.meta.url);
// @ts-expect-error The `path` module will anyway be imported by the injected script
globalThis.__dirname = path.dirname(__filename);


// src/bin/index.ts
import { parseArgs } from "node:util";

// src/bin/commands/init.ts
import childProcess from "node:child_process";
import fs2 from "node:fs/promises";
import path2 from "node:path";
import util from "node:util";
import json5 from "json5";
import ora from "ora";

// src/bin/utils/file.ts
import fs from "node:fs/promises";
import path from "node:path";
var exists = async (file) => {
  try {
    await fs.access(path.join(process.cwd(), file));
  } catch (_err) {
    return false;
  }
  return true;
};

// src/bin/commands/init.ts
var exec = util.promisify(childProcess.exec);
var init_default = async (positionals2) => {
  const spinner = ora("Initializing project").start();
  const lastPositional = positionals2[positionals2.length - 1];
  const spaceHandle = lastPositional === "init" ? null : lastPositional;
  if (!spaceHandle) {
    spinner.fail("Please provide a space handle like this:\n$ ronin init my-space");
    process.exit(1);
  }
  if (!await exists("package.json")) {
    spinner.fail(
      "No `package.json` found in the current directory. Please run the command in your project."
    );
    process.exit(1);
  }
  const packageManager = await exists("bun.lockb") ? "bun" : "npm";
  const packageManagerName = packageManager === "bun" ? "Bun" : "npm";
  spinner.text = `Detected ${packageManagerName} \u2014 installing types package`;
  try {
    const gitignoreExists = await exists(".gitignore");
    if (gitignoreExists) {
      const gitignorePath = path2.join(process.cwd(), ".gitignore");
      const gitignoreContents = await fs2.readFile(gitignorePath, "utf-8");
      if (!gitignoreContents.includes(".ronin")) {
        await fs2.appendFile(gitignorePath, "\n.ronin");
      }
    }
    if (packageManager === "bun") {
      await exec(`bun add @ronin/${spaceHandle} --dev`);
    } else {
      await exec(`npm install @ronin/${spaceHandle} --save-dev`);
    }
    const tsConfigPath = path2.join(process.cwd(), "tsconfig.json");
    const tsConfigExists = await exists("tsconfig.json");
    if (!tsConfigExists) return;
    const contents = await fs2.readFile(tsConfigPath, "utf-8");
    const tsConfig = json5.parse(contents);
    if (!tsConfig.compilerOptions.types?.includes(`@ronin/${spaceHandle}`))
      Object.assign(tsConfig.compilerOptions, {
        types: [...tsConfig.compilerOptions.types || [], `@ronin/${spaceHandle}`]
      });
    await fs2.writeFile(tsConfigPath, JSON.stringify(tsConfig, null, 2));
  } catch (err) {
    if (err instanceof Error && err.message.includes("401")) {
      spinner.fail(
        `You are not a member of the "${spaceHandle}" space or the space doesn't exist.`
      );
      process.exit(1);
    }
  }
  spinner.succeed("Project initialized");
};

// src/bin/commands/login.ts
import http from "node:http";
import getPort from "get-port";
import open from "open";
import ora2 from "ora";

// src/bin/utils/session.ts
import fs3 from "node:fs/promises";
import os from "node:os";
import path3 from "node:path";
import toml from "@iarna/toml";
import ini from "ini";
var CACHE_DIR = path3.join(os.homedir(), ".ronin");
var CACHE_DIR_FILE = path3.join(CACHE_DIR, "session.json");
var getSession = async () => {
  try {
    const contents = await fs3.readFile(CACHE_DIR_FILE, {
      encoding: "utf-8"
    });
    const parsedContents = JSON.parse(contents);
    if (parsedContents?.token) return parsedContents;
  } catch (err) {
    if (err.code !== "ENOENT") throw err;
  }
  return null;
};
var readConfigFile = async (filePath, name, parser) => {
  let npmConfigContents = {};
  try {
    const contents = await fs3.readFile(filePath, { encoding: "utf-8" });
    npmConfigContents = parser(contents);
  } catch (err) {
    if (err.code !== "ENOENT") {
      console.error(`Failed to read ${name} config file at ${filePath}`, err);
    }
  }
  return npmConfigContents;
};
var writeConfigFile = (filePath, contents) => {
  return fs3.writeFile(filePath, contents, { encoding: "utf-8" });
};
var storeSession = async (token) => {
  try {
    await fs3.stat(CACHE_DIR);
  } catch (err) {
    if (err.code === "ENOENT") {
      await fs3.mkdir(CACHE_DIR);
    } else {
      throw err;
    }
  }
  return writeConfigFile(CACHE_DIR_FILE, JSON.stringify({ token }, null, 2));
};
var storeTokenForNPM = async (token) => {
  const npmConfigFile = process.env.npm_config_userconfig || path3.join(os.homedir(), ".npmrc");
  const npmConfigContents = await readConfigFile(npmConfigFile, "npm", ini.parse);
  npmConfigContents["@ronin:registry"] = "https://ronin.supply";
  npmConfigContents["//ronin.supply/:_authToken"] = token;
  await writeConfigFile(npmConfigFile, ini.stringify(npmConfigContents));
};
var storeTokenForBun = async (token) => {
  const bunConfigFile = path3.join(os.homedir(), ".bunfig.toml");
  const bunConfigContents = await readConfigFile(bunConfigFile, "Bun", toml.parse);
  if (!bunConfigContents.install) bunConfigContents.install = {};
  if (!bunConfigContents.install.scopes) bunConfigContents.install.scopes = {};
  if (!bunConfigContents.install.scopes.ronin)
    bunConfigContents.install.scopes.ronin = {};
  bunConfigContents.install.scopes.ronin.url = "https://ronin.supply";
  bunConfigContents.install.scopes.ronin.token = token;
  await writeConfigFile(bunConfigFile, toml.stringify(bunConfigContents));
};

// src/bin/commands/login.ts
var logIn = async (appToken, exit = true) => {
  const spinner = ora2("Logging in").start();
  if (appToken) {
    await Promise.all([storeTokenForNPM(appToken), storeTokenForBun(appToken)]);
    spinner.succeed("Logged in successfully!");
    if (exit) process.exit(0);
    return;
  }
  let sessionToken;
  const port = await getPort();
  const server = http.createServer().listen(port);
  const baseURL = new URL("https://ronin.co/actions/auth/cli");
  const currentHost = `http://localhost:${port}`;
  const initURL = new URL(baseURL);
  initURL.searchParams.set("from", currentHost);
  spinner.text = `Please log in using the following URL:
${initURL.href}`;
  spinner.suffixText = "\n";
  try {
    [sessionToken] = await Promise.all([
      new Promise((resolve, reject) => {
        server.once("request", (req, res) => {
          res.setHeader("connection", "close");
          const requestURL = new URL(req.url || "", currentHost);
          const requestToken = requestURL.searchParams.get("token");
          if (!requestToken) return reject(new Error("Missing token"));
          resolve(requestToken);
          const headers = { Location: baseURL.href };
          res.writeHead(307, headers).end();
        });
        server.once("error", reject);
      }),
      open(initURL.href)
    ]);
  } finally {
    server.close();
  }
  await Promise.all([
    storeSession(sessionToken),
    storeTokenForNPM(sessionToken),
    storeTokenForBun(sessionToken)
  ]);
  spinner.suffixText = "";
  spinner.succeed("Logged in successfully!");
  if (exit) process.exit(0);
};
var login_default = logIn;

// src/bin/commands/sync.ts
import path6 from "node:path";
import { select } from "@inquirer/prompts";
import ora3 from "ora";

// src/bin/parser/index.ts
import fs4 from "node:fs/promises";
import path4 from "node:path";
import ts2 from "typescript";

// src/bin/parser/errors.ts
import pluralize from "pluralize";
var ADVANCED_FIELD_TYPES = ["Blob", "JSON"];
var BASIC_FIELD_TYPES = ["string", "number", "boolean", "Date"];
function createMissingSchemaError(missingSchemas) {
  return `The following schemas were used as a reference but weren't included in the \`Schemas\` interface:

${missingSchemas.map(({ name, parent, source }) => `  - \`${name}\` in \`${parent}\` (${source})`).join(
    "\n"
  )}

Please include them in the \`Schemas\` interface or remove their references.`;
}
function createMissingPluralError(schemaName, schemaSlug, pluralSchemaSlug, schemaRecordsAlias) {
  const pluralTypeName = pluralize(schemaName);
  return (
    // biome-ignore lint/style/useTemplate: We do this because we don't want to have too long lines.
    `The schema \`${schemaName}\` does not have a plural slug and name defined.

Please define them in your schema definition file and include them in the \`Schemas\` interface:

import type * as Schema from 'ronin/schema';

type ${pluralTypeName} = Schema.${schemaRecordsAlias}<${schemaName}>;

interface Schemas {
  ${schemaSlug}: ${schemaName};
  ${pluralize(pluralSchemaSlug)}: ${pluralTypeName};
}`
  );
}
function createUnknownFieldError(unknownFields) {
  return (
    // biome-ignore lint/style/useTemplate: We do this because we don't want to have too long lines.
    `The type of the following fields could not be determined:

${unknownFields.map(
      ({ name, parent, type, source }) => `  - \`${parent}.${name}\` is typed as \`${type}\` (${source})`
    ).join("\n")}

Please make sure that the field is typed as any of the available field types:

${BASIC_FIELD_TYPES.map((type) => `  - \`${type}\``).join("\n")}
${ADVANCED_FIELD_TYPES.map((type) => `  - \`Schema.${type}\``).join("\n")}
  - or a reference to another schema.`
  );
}

// src/bin/parser/utils.ts
import ts from "typescript";

// src/bin/utils/env.ts
var isTest = () => process.env.NODE_ENV === "test";

// src/utils/id.ts
import { init } from "@paralleldrive/cuid2";
var generateUniqueId = (length = 24) => init({ length })();

// src/bin/parser/utils.ts
var generateFieldId = (type) => {
  const id = isTest() ? "0" : generateUniqueId();
  return `${type}-${id}`;
};
function convertToReadableText(str) {
  if (str === "") return "";
  if (!str || typeof str !== "string") return null;
  let spacedStr = str.replace(/([a-z])([A-Z])/g, "$1 $2");
  spacedStr = spacedStr.replace(/([a-zA-Z])(\d)/g, "$1 $2").replace(/(\d)([a-zA-Z])/g, "$1 $2");
  const words = spacedStr.split(" ");
  const capitalizedWords = words.map(
    (word) => word.charAt(0).toUpperCase() + word.slice(1)
  );
  return capitalizedWords.join(" ");
}
function getLineAndColumnsNumber(node, sourceFile) {
  const { line: startLine, character: startCharacter } = sourceFile.getLineAndCharacterOfPosition(node.getStart());
  const { line: endLine, character: endCharacter } = sourceFile.getLineAndCharacterOfPosition(node.getEnd());
  return {
    start: { line: startLine + 1, character: startCharacter + 1 },
    end: { line: endLine + 1, character: endCharacter + 1 }
  };
}
function parseJsDoc(node) {
  let name = "";
  let description = "";
  const details = {};
  const commentsAndTags = ts.getJSDocCommentsAndTags(node);
  if (commentsAndTags.length > 0) {
    const jsDoc = commentsAndTags[0];
    const commentText = jsDoc.comment ? ts.getTextOfJSDocComment(jsDoc.comment) : void 0;
    if (commentText) {
      const lines = commentText.split("\n").map((line) => line.trim()).filter((line) => line);
      if (lines.length > 0) {
        name = lines[0];
        description = lines.slice(1).join(" ");
      }
    }
    if ("tags" in jsDoc && jsDoc.tags) {
      for (const tag of jsDoc.tags) {
        const tagName = tag.tagName.getText();
        const tagComment = ts.getTextOfJSDocComment(tag.comment);
        if (tagComment) {
          details[tagName] = tagComment;
        }
      }
    }
  }
  return { name, description, details };
}

// src/bin/parser/index.ts
var EXPERIMENTAL = {
  TSDOC_TAGS: isTest(),
  JSON_FIELD_STRUCTURE: isTest()
};
var RONIN_MODULE = "ronin";
var SCHEMAS_INTERFACE = "Schemas";
async function parseSchemaDefinitionFile(filePath = "./schemas/index.ts") {
  const relativePath = path4.relative(process.cwd(), filePath);
  const fullPath = path4.resolve(process.cwd(), filePath);
  const schemaFileExists = await exists(filePath);
  if (!schemaFileExists) {
    throw new Error(
      `The given path to the schema definition file does not exist: ${fullPath}`
    );
  }
  const fileContent = await fs4.readFile(fullPath, "utf-8");
  return parseSchemaDefinitions(fileContent, relativePath);
}
function parseSchemaDefinitions(content, relativePath) {
  const sourceFile = ts2.createSourceFile(
    "temp.ts",
    content,
    ts2.ScriptTarget.Latest,
    true
  );
  const schemas = [];
  const typeMapping = /* @__PURE__ */ new Map();
  const namespaces = /* @__PURE__ */ new Map();
  const registeredSchemas = /* @__PURE__ */ new Map();
  const missingSchemas = [];
  const unknownFields = [];
  let schemaRecordAlias = "Record";
  let schemaRecordsAlias = "Records";
  function getFieldType(type) {
    const originalType = typeMapping.get(type) || type;
    switch (originalType) {
      case "string":
      case "number":
      case "boolean":
        return originalType;
      case "Date":
        return "date";
      case "Blob":
        return "blob";
      case "JSON":
        return "json";
      default: {
        const { resolvedTypeName } = resolveTypeAlias(originalType);
        if (resolvedTypeName === schemaRecordAlias) {
          return "reference";
        }
        return "unknown";
      }
    }
  }
  function visitImports(node) {
    if (ts2.isImportDeclaration(node) && node.importClause && node.importClause.namedBindings) {
      const namedBindings = node.importClause.namedBindings;
      if (ts2.isNamedImports(namedBindings)) {
        for (const element of namedBindings.elements) {
          const importedName = element.name.text;
          const originalName = element.propertyName ? element.propertyName.text : importedName;
          typeMapping.set(importedName, originalName);
          if (originalName === "Record") {
            schemaRecordAlias = importedName;
          } else if (originalName === "Records") {
            schemaRecordsAlias = importedName;
          }
        }
      } else if (ts2.isNamespaceImport(namedBindings)) {
        namespaces.set(
          namedBindings.name.text,
          node.moduleSpecifier.getText().replace(/['"]/g, "")
        );
      }
    }
  }
  function visitRoninModuleDeclaration(node) {
    if (ts2.isModuleDeclaration(node) && node.name.text === RONIN_MODULE) {
      if (node.body && ts2.isModuleBlock(node.body)) {
        for (const statement of node.body.statements) {
          if (ts2.isInterfaceDeclaration(statement) && statement.name.text === SCHEMAS_INTERFACE) {
            visitInterface(statement);
          }
        }
      }
    }
  }
  function visitInterface(node) {
    for (const member of node.members) {
      if (ts2.isPropertySignature(member)) {
        const propertyName = member.name.getText();
        const typeName = member.type && ts2.isTypeReferenceNode(member.type) ? member.type.typeName.getText() : "";
        if (typeName) {
          registeredSchemas.set(propertyName, typeName);
        }
      }
    }
    for (const [propertyName, typeAlias] of registeredSchemas) {
      const parsedType = parseSchemaProperty(propertyName, typeAlias);
      if (parsedType) {
        schemas.push(parsedType);
      }
    }
  }
  function resolveTypeAlias(typeName) {
    let resolvedTypeName = typeName;
    let typeArguments = [];
    let namespace = null;
    function typeAliasVisitor(node) {
      if (ts2.isTypeAliasDeclaration(node) && node.name.text === typeName) {
        if (ts2.isTypeReferenceNode(node.type) && node.type.typeName) {
          const fullTypeName = node.type.typeName.getText();
          const parts = fullTypeName.split(".");
          if (parts.length > 1) {
            namespace = parts.slice(0, -1).join(".");
            resolvedTypeName = parts[parts.length - 1];
          } else {
            resolvedTypeName = fullTypeName;
          }
          if (node.type.typeArguments) {
            typeArguments = Array.from(node.type.typeArguments);
          }
        }
      }
      ts2.forEachChild(node, typeAliasVisitor);
    }
    ts2.forEachChild(sourceFile, typeAliasVisitor);
    return { resolvedTypeName, typeArguments, namespace };
  }
  function parseTypeNode(typeNode) {
    if (ts2.isTypeLiteralNode(typeNode)) {
      return typeNode.members.map((member) => {
        if (ts2.isPropertySignature(member)) {
          if (!member.type) {
            throw new Error(
              `Property signature \`${member.getText()}\` in ${typeNode.parent.getText()} is missing a type`
            );
          }
          const fieldName = member.name.getText();
          const isRequired = !member.questionToken;
          let fieldType = getFieldType(member.type.getText());
          const parsedField = {
            name: fieldName,
            type: fieldType,
            required: isRequired
          };
          if (ts2.isArrayTypeNode(member.type)) {
            parsedField.type = "array";
            parsedField.children = parseTypeNode(member.type.elementType);
          } else if (ts2.isUnionTypeNode(member.type)) {
            parsedField.type = "enum";
            parsedField.value = member.type.types.map(
              (t) => t.getText().replace(/"/g, "")
            );
          } else if (ts2.isTypeReferenceNode(member.type)) {
            const typeName = member.type.typeName.getText();
            if (typeName === "Array") {
              parsedField.type = "array";
              parsedField.children = parseTypeNode(member.type.typeArguments[0]);
            } else {
              fieldType = getFieldType(typeName);
              parsedField.type = fieldType;
              if (member.type.typeArguments) {
                parsedField.meta = parseTypeNode(member.type.typeArguments[0]);
              }
            }
          } else if (ts2.isTypeLiteralNode(member.type)) {
            parsedField.type = "object";
            parsedField.children = parseTypeNode(member.type);
          } else if (ts2.isLiteralTypeNode(member.type)) {
            parsedField.type = "enum";
            parsedField.value = [member.type.literal.getText().replace(/"/g, "")];
          } else {
            parsedField.type = fieldType;
          }
          return parsedField;
        }
        throw new Error(
          `Unsupported property type \`${member.getText()}\` in ${typeNode.parent.getText()}`
        );
      });
    }
    if (ts2.isUnionTypeNode(typeNode)) {
      return {
        type: "enum",
        value: typeNode.types.map((t) => t.getText().replace(/"/g, ""))
      };
    }
    if (ts2.isArrayTypeNode(typeNode)) {
      return {
        type: "array",
        children: parseTypeNode(typeNode.elementType)
      };
    }
    if (ts2.isTypeReferenceNode(typeNode)) {
      const typeName = typeNode.typeName.getText();
      return {
        name: typeName,
        type: getFieldType(typeName),
        ...typeNode.typeArguments ? { meta: parseTypeNode(typeNode.typeArguments[0]) } : {}
      };
    }
    if (ts2.isLiteralTypeNode(typeNode)) {
      return {
        type: "enum",
        value: [typeNode.literal.getText().replace(/"/g, "")]
      };
    }
    return { type: typeNode.getText() };
  }
  function checkSchemaInclusion(node, parent) {
    const nodeType = node.type;
    const typeName = nodeType.getText();
    const schemaSlug = Array.from(registeredSchemas.keys()).find(
      (key) => registeredSchemas.get(key) === typeName
    );
    if (schemaSlug) {
      return schemaSlug;
    }
    const source = getLineAndColumnsNumber(node, sourceFile);
    missingSchemas.push({
      name: typeName,
      parent,
      source: `${relativePath}:${source.start.line}:${source.start.character}`
    });
    return null;
  }
  function getTypeFromNamespace(typeName) {
    const [namespace, type] = typeName.split(".");
    if (namespaces.get(namespace)) {
      return typeMapping.get(type) || type;
    }
    return typeName;
  }
  function parseFieldDefinition(node, parent) {
    if (!node.type) {
      throw new Error(
        `Property signature \`${node.getText()}\` in ${parent} is missing a type`
      );
    }
    const fieldName = node.name.getText();
    const memberType = ts2.isTypeReferenceNode(node.type) ? node.type.typeName.getText() : node.type.getText();
    const memberTypeText = getTypeFromNamespace(memberType);
    const fieldType = getFieldType(memberTypeText);
    const { name, description, details } = parseJsDoc(node);
    if (fieldType === "unknown") {
      const source = getLineAndColumnsNumber(node, sourceFile);
      unknownFields.push({
        parent,
        name: fieldName,
        type: memberTypeText,
        source: `${relativePath}:${source.start.line}:${source.start.character}`
      });
      return null;
    }
    const field = {
      id: generateFieldId(fieldType),
      type: fieldType,
      slug: fieldName,
      name: name || fieldName.charAt(0).toUpperCase() + fieldName.slice(1),
      unique: false,
      required: !node.questionToken
    };
    if (description) {
      field.description = description;
    }
    if (field.type === "string") {
      field.displayAs = "single-line";
    }
    if (field.type === "reference") {
      let schema = null;
      schema = checkSchemaInclusion(node, parent);
      if (schema) {
        field.schema = schema;
      }
    }
    if (EXPERIMENTAL.TSDOC_TAGS && Object.keys(details).length > 0) {
      field.details = details;
    }
    if (EXPERIMENTAL.JSON_FIELD_STRUCTURE && field.type === "json" && node.type && ts2.isTypeReferenceNode(node.type) && node.type.typeArguments) {
      field.meta = parseTypeNode(node.type.typeArguments[0]);
    }
    return field;
  }
  function parseSchemaProperty(propertyName, typeAlias) {
    const result = {
      name: convertToReadableText(typeAlias),
      slug: propertyName,
      pluralName: "",
      pluralSlug: "",
      fields: []
    };
    for (const [schemaSlug, schemaType] of registeredSchemas) {
      const { resolvedTypeName, typeArguments } = resolveTypeAlias(schemaType);
      const firstTypeArgument = typeArguments[0]?.getText();
      if (resolvedTypeName === schemaRecordsAlias && typeAlias === firstTypeArgument) {
        result.pluralName = convertToReadableText(schemaType);
        result.pluralSlug = schemaSlug;
      }
    }
    function typeAliasVisitor(node) {
      if (ts2.isTypeAliasDeclaration(node) && node.name.text === typeAlias) {
        if (ts2.isTypeReferenceNode(node.type) && node.type.typeArguments) {
          const typeLiteral = node.type.typeArguments[0];
          if (ts2.isTypeLiteralNode(typeLiteral)) {
            for (const member of typeLiteral.members) {
              if (ts2.isPropertySignature(member)) {
                const parsedField = parseFieldDefinition(member, typeAlias);
                if (parsedField) {
                  result.fields.push(parsedField);
                }
              }
            }
          }
        }
      }
      ts2.forEachChild(node, typeAliasVisitor);
    }
    ts2.forEachChild(sourceFile, typeAliasVisitor);
    if (!result.pluralName && result.fields.length) {
      throw new Error(
        createMissingPluralError(
          typeAlias,
          propertyName,
          result.slug,
          schemaRecordsAlias
        )
      );
    }
    return result.fields.length ? result : null;
  }
  function visit(node) {
    visitImports(node);
    visitRoninModuleDeclaration(node);
    ts2.forEachChild(node, visit);
  }
  visit(sourceFile);
  if (missingSchemas.length > 0) {
    throw new Error(createMissingSchemaError(missingSchemas));
  }
  if (unknownFields.length > 0) {
    throw new Error(createUnknownFieldError(unknownFields));
  }
  return schemas;
}

// src/bin/utils/config.ts
import fs5 from "node:fs";
import path5 from "node:path";
var saveConfig = (config) => {
  const configDir = path5.join(process.cwd(), ".ronin");
  if (!fs5.existsSync(configDir)) {
    fs5.mkdirSync(configDir);
  }
  const configPath = path5.join(configDir, "config.json");
  let existingConfig = {};
  if (fs5.existsSync(configPath)) {
    existingConfig = JSON.parse(fs5.readFileSync(configPath, "utf-8"));
  }
  fs5.writeFileSync(configPath, JSON.stringify({ ...existingConfig, ...config }, null, 2));
};
var resetConfig = () => {
  const configPath = path5.join(process.cwd(), ".ronin", "config.json");
  if (fs5.existsSync(configPath)) {
    fs5.unlinkSync(configPath);
  }
};
var readConfig = () => {
  const configPath = path5.join(process.cwd(), ".ronin", "config.json");
  if (!fs5.existsSync(configPath)) {
    return {};
  }
  return JSON.parse(fs5.readFileSync(configPath, "utf-8"));
};

// src/bin/utils/json.ts
var safeParseJson = (json) => {
  try {
    return JSON.parse(json);
  } catch {
    return json;
  }
};

// src/bin/utils/sync.ts
import { confirm } from "@inquirer/prompts";
var getSpaces = async (sessionToken) => {
  let text;
  try {
    const res = await fetch("https://ronin.supply/-/ronin/spaces", {
      method: "GET",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${sessionToken}`
      }
    });
    text = await res.text();
    if (!res.ok) {
      throw new Error(text);
    }
  } catch (err) {
    throw new Error(`Failed to fetch available spaces: ${err.message}`);
  }
  let json;
  try {
    json = JSON.parse(text);
  } catch (_) {
    throw new Error(`Failed to parse response from server: ${text}`);
  }
  return json.spaces;
};
var getSchemas = async (token, space) => {
  let text;
  try {
    const res = await fetch(
      `https://ronin.supply/-/ronin/schemas?data-selector=${space}`,
      {
        method: "GET",
        headers: {
          "Content-Type": "application/json",
          Authorization: `Bearer ${token}`
        }
      }
    );
    text = await res.text();
    if (!res.ok) {
      throw new Error(text);
    }
  } catch (err) {
    throw new Error(`Failed to fetch remote schemas: ${err.message}`);
  }
  let json;
  try {
    json = JSON.parse(text);
  } catch (_) {
    throw new Error(`Failed to parse response from server: ${text}`);
  }
  return json.schemas;
};
var replaceFieldIdsWithExisting = async (local, remote) => {
  return local.map((schema) => {
    const existingSchema = remote.find((s) => s.slug === schema.slug);
    if (!existingSchema) {
      return schema;
    }
    const existingFields = existingSchema.fields || [];
    const localFields = schema.fields || [];
    const fields = localFields.map((field) => {
      const existingField = existingFields.find((f) => f.slug === field.slug);
      if (!existingField) {
        return field;
      }
      return {
        ...field,
        id: existingField.id
      };
    });
    return {
      ...schema,
      fields
    };
  });
};
var compareSchemas = async (local, remote, spinner) => {
  const fieldsWithDifferentTypes = [];
  for (const { slug: schemaSlug, fields } of local) {
    if (!fields) return;
    const existingSchema = remote.find((schema) => schema.slug === schemaSlug);
    const existingFields = existingSchema?.fields || [];
    if (!existingSchema) {
      return;
    }
    for (const field of fields) {
      const existingField = existingFields.find((f) => f.slug === field.slug);
      if (!existingField) {
        return;
      }
      if (existingField.type !== field.type) {
        field.id = generateFieldId(field.type);
        fieldsWithDifferentTypes.push(
          `${schemaSlug}.${field.slug} (${existingField.type} -> ${field.type})`
        );
      }
    }
  }
  if (fieldsWithDifferentTypes.length === 0) {
    return;
  }
  spinner.warn(
    `The data type of the following fields have changed:
${fieldsWithDifferentTypes.map((field) => `  - ${field}`).join("\n")}

`
  );
  const answer = await confirm({
    message: "The content of these fields will be lost. Do you want to continue?",
    default: false
  });
  if (!answer) {
    spinner.stop();
    process.exit();
  }
};

// src/bin/commands/sync.ts
var DEFAULT_SCHEMA_SUMMARY = "This is a newly created schema.";
var sync_default = async (positionals2, appToken, sessionToken, reset = false) => {
  if (reset) {
    resetConfig();
  }
  let status = "readingConfig";
  const spinner = ora3("Reading configuration").start();
  const token = appToken || sessionToken;
  if (!token) {
    throw new Error("Either an app token or session token is required to sync schemas.");
  }
  const customDir = positionals2[0] && !positionals2[0].startsWith("-") ? positionals2[0] : void 0;
  if (customDir) {
    const schemaFile = path6.join(customDir, "index.ts");
    const schemaFileExists = await exists(schemaFile);
    if (!schemaFileExists) {
      spinner.fail(
        "The provided schema directory does not exist or does not contain a schema definitions file (`index.ts`)."
      );
      process.exit(1);
    }
    saveConfig({ schemasDir: customDir });
  }
  const config = readConfig();
  try {
    if (!(config.spaceId || appToken) && sessionToken) {
      const spaces = await getSpaces(sessionToken);
      if (!spaces || spaces.length === 0) {
        spinner.fail(
          "You don't have access to any space or your CLI session is invalid.\n\nPlease login again (by running `npx ronin login`) or create a new space on the dashboard (`https://ronin.co/new`) and try again."
        );
        process.exit(1);
      }
      if (spaces.length === 1) {
        config.spaceId = spaces[0].id;
        saveConfig({ spaceId: spaces[0].id });
      } else {
        spinner.stop();
        const answer = await select({
          message: "Which space do you want to sync schemas to?",
          choices: spaces.map((space) => ({
            name: space.handle,
            value: space.id,
            description: space.name
          }))
        });
        config.spaceId = answer;
        saveConfig({ spaceId: answer });
      }
    }
    status = "readingSchemas";
    spinner.text = "Reading schema definitions";
    const schemaFile = path6.join(config.schemasDir || "schemas", "index.ts");
    let schemaDefinitions = await parseSchemaDefinitionFile(schemaFile);
    status = "comparing";
    spinner.start("Retrieving existing schemas");
    const remoteSchemas = await getSchemas(token, config.spaceId);
    schemaDefinitions = await replaceFieldIdsWithExisting(
      schemaDefinitions,
      remoteSchemas
    );
    schemaDefinitions = schemaDefinitions.map((schema) => ({
      ...schema,
      summary: remoteSchemas.find(({ slug }) => slug === schema.slug)?.summary || DEFAULT_SCHEMA_SUMMARY
    }));
    status = "comparing";
    spinner.text = "Comparing local and remote schema definitions";
    await compareSchemas(schemaDefinitions, remoteSchemas, spinner);
    status = "syncing";
    spinner.text = "Syncing schema definitions";
    const res = await fetch("https://ronin.supply/-/ronin/sync", {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${appToken || sessionToken}`
      },
      body: JSON.stringify({
        schemas: schemaDefinitions,
        space: config.spaceId
      })
    });
    const text = await res.text();
    if (!res.ok) {
      throw new Error(text);
    }
  } catch (err) {
    spinner.fail(
      `Failed to ${status === "readingSchemas" ? "read schema definitions" : "apply new schema changes"}:
`
    );
    if (err instanceof Error) {
      const errorObject = safeParseJson(err.message);
      if (typeof errorObject === "object" && "error" in errorObject) {
        console.error(errorObject.error);
      } else {
        console.error(errorObject);
      }
    } else {
      console.error(err);
    }
    process.exit(1);
  }
  spinner.succeed("Schemas are in sync!");
};

// src/bin/utils/info.ts
import chalkTemplate from "chalk-template";

// package.json
var version = "4.4.1-dependabot-interval-experimental.0";

// src/bin/utils/info.ts
var printVersion = () => {
  console.log(version);
  process.exit(0);
};
var printHelp = () => {
  const text = chalkTemplate`
  {bold.magenta ronin} — Data at the edge

  {bold USAGE}

      {bold $} {bold.magenta ronin}
      {bold $} {bold.magenta ronin} login
      {bold $} {bold.magenta ronin} --help
      {bold $} {bold.magenta ronin} --version

  {bold COMMANDS}

      login                               Authenticate with RONIN (run by default for every command)
      init [space]                        Initialize the TypeScript types for a given space
      sync [dir]                          Read local schema definitions (from a given directory) and sync them with RONIN

  {bold OPTIONS}

      -h, --help                          Shows this help message
      -v, --version                       Shows the version of the CLI that is currently installed
      -d, --debug                         Shows additional debugging information
  `;
  console.log(text);
  process.exit(0);
};

// src/bin/index.ts
var values;
var positionals;
try {
  ({ values, positionals } = parseArgs({
    args: process.argv,
    options: {
      help: {
        type: "boolean",
        short: "h",
        default: false
      },
      version: {
        type: "boolean",
        short: "v",
        default: false
      },
      debug: {
        type: "boolean",
        short: "d",
        default: false
      },
      reset: {
        type: "boolean",
        short: "r",
        default: false
      }
    },
    strict: true,
    allowPositionals: true
  }));
} catch (err) {
  if (err instanceof Error) {
    console.error(err.message);
  } else {
    throw err;
  }
  process.exit(1);
}
var run = async () => {
  if (values.help) return printHelp();
  if (values.version) return printVersion();
  const normalizedPositionals = positionals.map((positional) => positional.toLowerCase());
  const appToken = process.env.RONIN_TOKEN;
  const session = await getSession();
  if (!(process.stdout.isTTY || session || appToken)) {
    let message = "If RONIN CLI is invoked from a non-interactive shell, ";
    message += "a `RONIN_TOKEN` environment variable containing an app token must be provided.";
    console.error(message);
    process.exit(1);
  }
  if (!(session || normalizedPositionals.includes("login"))) await login_default(appToken, false);
  if (normalizedPositionals.includes("login")) return login_default(appToken);
  if (normalizedPositionals.includes("init")) return init_default(positionals);
  if (normalizedPositionals.includes("sync")) {
    const reset = values.reset;
    return sync_default(
      positionals.slice(positionals.indexOf("sync") + 1),
      appToken,
      session?.token,
      reset
    );
  }
  return printHelp();
};
run();
process.on("SIGINT", () => {
  process.exit(0);
});
process.on("SIGTERM", () => {
  process.exit(0);
});
