// src/utils/errors.ts
var InvalidQueryError = class extends Error {
  message;
  query;
  path;
  details;
  code;
  fields;
  constructor(details) {
    super(details.message);
    this.name = "InvalidQueryError";
    this.message = details.message;
    this.query = details.query;
    this.path = details.path;
    this.details = details.details;
    this.code = details.code;
    this.fields = details.fields;
  }
};
var InvalidResponseError = class extends Error {
  message;
  code;
  constructor(details) {
    super(details.message);
    this.name = "InvalidResponseError";
    this.message = details.message;
    this.code = details.code;
  }
};
var getResponseBody = async (response, options) => {
  if (response.ok) return response.json();
  const text = await response.text();
  let json;
  try {
    json = JSON.parse(text);
  } catch (_err) {
    throw new InvalidResponseError({
      message: `${options?.errorPrefix ? `${options.errorPrefix} ` : ""}${text}`,
      code: "JSON_PARSE_ERROR"
    });
  }
  if (json.error) {
    json.error.message = `${options?.errorPrefix ? `${options.errorPrefix} ` : ""}${json.error.message}`;
    throw new InvalidResponseError(json.error);
  }
  return json;
};
var getDotNotatedPath = (segments = []) => segments.length > 0 ? segments.reduce((path, segment, index) => {
  if (typeof segment === "number") return `${path}[${segment}]`;
  if (index === 0) return `${segment}`;
  return `${path}.${segment}`;
}, "") : null;

// src/storage.ts
var extractStorableObjects = (queries) => queries.reduce(
  (references, query, queryIndex) => {
    return [
      // biome-ignore lint/performance/noAccumulatingSpread: This code is too complex to refactor.
      ...references,
      ...Object.entries(query).reduce(
        (references2, [queryType, query2]) => {
          if (!["set", "create"].includes(queryType)) return references2;
          return [
            // biome-ignore lint/performance/noAccumulatingSpread: This code is too complex to refactor.
            ...references2,
            ...Object.entries(query2).reduce(
              (references3, [schema, instructions]) => {
                const fields = instructions[queryType === "set" ? "to" : "with"];
                return [
                  // biome-ignore lint/performance/noAccumulatingSpread: This code is too complex to refactor.
                  ...references3,
                  ...Object.entries(fields).reduce(
                    (references4, [name, value]) => {
                      const isStorableObject = typeof File !== "undefined" && value instanceof File || typeof ReadableStream !== "undefined" && value instanceof ReadableStream || typeof Blob !== "undefined" && value instanceof Blob || typeof ArrayBuffer !== "undefined" && value instanceof ArrayBuffer || typeof Buffer !== "undefined" && Buffer.isBuffer(value);
                      if (!isStorableObject) return references4;
                      const storarableObject = {
                        query: {
                          index: queryIndex,
                          type: queryType
                        },
                        schema,
                        field: name,
                        value
                      };
                      if ("type" in value) {
                        storarableObject.contentType = value.type;
                      }
                      if ("name" in value) {
                        storarableObject.name = value.name;
                      }
                      return [...references4, storarableObject];
                    },
                    []
                  )
                ];
              },
              []
            )
          ];
        },
        []
      )
    ];
  },
  []
);
var uploadStorableObjects = (storableObjects, options = {}) => {
  const fetcher = typeof options?.fetch === "function" ? options.fetch : fetch;
  const requests = storableObjects.map(
    async ({ name, value, contentType }) => {
      const headers = new Headers();
      headers.set("Authorization", `Bearer ${options.token}`);
      if (contentType) {
        headers.set("Content-Type", contentType);
      }
      if (name) {
        headers.set(
          "Content-Disposition",
          `form-data; filename="${encodeURIComponent(name)}"`
        );
      }
      const request = new Request("https://storage.ronin.co/", {
        method: "PUT",
        body: value,
        headers
      });
      const response = await fetcher(request);
      return getResponseBody(response, {
        errorPrefix: "An error occurred while uploading the binary objects included in the provided queries. Error:"
      });
    }
  );
  return Promise.all(requests);
};
var processStorableObjects = async (queries, upload) => {
  const objects = extractStorableObjects(queries);
  if (objects.length > 0) {
    const storedObjects = await upload(objects);
    for (let index = 0; index < objects.length; index++) {
      const { query, schema, field } = objects[index];
      const reference = storedObjects[index];
      queries[query.index][query.type][schema][query.type === "set" ? "to" : "with"][field] = reference;
    }
  }
  return queries;
};

// src/utils/helpers.ts
var objectFromAccessor = (accessor, value) => setProperty({}, accessor, value);
var getPathSegments = (path) => {
  const segments = path.replace(/\\\./g, "\u200B").split(/[[.]/g).map((s) => s.replace(/\u200B/g, ".")).filter((x) => !!x.trim()).map((x) => x.replaceAll("\\.", "."));
  return segments;
};
var setPropertyViaPathSegments = (obj, pathSegments, value) => {
  let current = obj;
  for (let i = 0; i < pathSegments.length; i++) {
    const key = pathSegments[i];
    const isLastKey = i === pathSegments.length - 1;
    if (isLastKey) {
      current[key] = typeof value === "function" ? value(current[key]) : value;
    } else {
      if (!Object.prototype.hasOwnProperty.call(current, key) || typeof current[key] !== "object") {
        current[key] = {};
      }
      current = current[key];
    }
  }
};
var setProperty = (obj, path, value) => {
  const segments = getPathSegments(path);
  setPropertyViaPathSegments(obj, segments, value);
  return obj;
};
var getPropertyViaPathSegments = (obj, pathSegments) => {
  let current = obj;
  for (const key of pathSegments) {
    if (current[key] === null || current[key] === void 0) return void 0;
    current = current[key];
  }
  return current;
};
var getProperty = (obj, path) => getPropertyViaPathSegments(obj, getPathSegments(path));
var toDashCase = (string) => {
  const capitalize = (str) => {
    const lower = str.toLowerCase();
    return lower.substring(0, 1).toUpperCase() + lower.substring(1, lower.length);
  };
  const parts = string?.replace(/([A-Z])+/g, capitalize)?.split(/(?=[A-Z])|[.\-\s_]/).map((x) => x.toLowerCase()) ?? [];
  if (parts.length === 0) return "";
  if (parts.length === 1) return parts[0];
  return parts.reduce((acc, part) => `${acc}-${part.toLowerCase()}`);
};
var formatDateFields = (record, dateFields) => {
  for (const field of dateFields) {
    setProperty(
      record,
      field,
      (value) => value !== null ? new Date(value) : null
    );
  }
};
var mergeOptions = (...options) => {
  return options.reduce((acc, opt) => {
    const resolvedOpt = typeof opt === "function" ? opt() : opt;
    Object.assign(acc, resolvedOpt);
    return acc;
  }, {});
};

// src/utils/data-hooks.ts
var EMPTY = Symbol("empty");
var HOOK_TYPES = ["before", "during", "after"];
var getSchema = (instruction) => {
  const key = Object.keys(instruction)[0];
  let schema = String(key);
  let multipleRecords = false;
  if (schema.endsWith("s")) {
    schema = schema.substring(0, schema.length - 1);
    multipleRecords = true;
  }
  return {
    key,
    // Convert camel case (e.g. `subscriptionItems`) into slugs
    // (e.g. `subscription-items`).
    schema: toDashCase(schema),
    multipleRecords
  };
};
var getMethodName = (hookType, queryType) => {
  const capitalizedQueryType = queryType[0].toUpperCase() + queryType.slice(1);
  return hookType === "during" ? queryType : hookType + capitalizedQueryType;
};
var normalizeResults = (result) => {
  const value = Array.isArray(result) ? result : result === EMPTY ? [] : [result];
  return structuredClone(value);
};
var invokeHooks = async (hookType, query, options) => {
  const { hooks, asyncContext } = options;
  const queryType = Object.keys(query.definition)[0];
  const queryInstructions = query.definition[queryType];
  const { key, schema: querySchema, multipleRecords } = getSchema(queryInstructions);
  const oldInstruction = queryInstructions[key];
  const hooksForSchema = hooks[querySchema];
  const hookName = getMethodName(hookType, queryType);
  const queryInstruction = oldInstruction ? structuredClone(oldInstruction) : {};
  const parentHook = asyncContext.getStore();
  const shouldSkip = hooksForSchema && (hooksForSchema.get || hooksForSchema.count || hooksForSchema.create || hooksForSchema.set || hooksForSchema.drop) && parentHook && querySchema !== parentHook.querySchema ? false : parentHook && (HOOK_TYPES.indexOf(hookType) <= HOOK_TYPES.indexOf(parentHook.hookType) || querySchema === parentHook.querySchema && HOOK_TYPES.indexOf(hookType) > HOOK_TYPES.indexOf(parentHook.hookType));
  if (hooksForSchema && hookName in hooksForSchema && !shouldSkip) {
    const hook = hooksForSchema[hookName];
    const hookResult = await asyncContext.run(
      {
        hookType,
        queryType,
        querySchema
      },
      async () => {
        if (hookType === "after") {
          return hook(
            queryInstruction,
            multipleRecords,
            normalizeResults(query.resultBefore),
            normalizeResults(query.resultAfter)
          );
        }
        return hook(
          queryInstruction,
          multipleRecords
        );
      }
    );
    if (hookType === "before") {
      queryInstructions[key] = hookResult;
      return { definition: { [queryType]: queryInstructions }, result: EMPTY };
    }
    if (hookType === "during") {
      return { definition: query.definition, result: hookResult };
    }
  }
  return { definition: query.definition, result: EMPTY };
};
var runQueriesWithHooks = async (queries, options = {}) => {
  const { hooks, waitUntil, asyncContext } = options;
  if (!hooks) return runQueries(queries, options);
  if (typeof process === "undefined" && !waitUntil) {
    let message = 'In the case that the "ronin" package receives a value for';
    message += " its `hooks` option, it must also receive a value for its";
    message += " `waitUntil` option. This requirement only applies when using";
    message += " an edge runtime and ensures that the edge worker continues to";
    message += ' execute until all "after" hooks have been executed.';
    throw new Error(message);
  }
  if (!asyncContext) {
    let message = 'In the case that the "ronin" package receives a value for';
    message += " its `hooks` option, it must also receive a value for its";
    message += " `asyncContext` option.";
    throw new Error(message);
  }
  const queryList = queries.flatMap((query, index) => {
    const details = { definition: query, result: EMPTY };
    if (query.set) {
      const schemaSlug = Object.keys(query.set)[0];
      const diffQuery = {
        definition: {
          get: {
            [schemaSlug]: {
              with: query.set[schemaSlug].with
            }
          }
        },
        diffForIndex: index + 1,
        result: EMPTY
      };
      return [diffQuery, details];
    }
    return [details];
  });
  const hookCallerOptions = { hooks, asyncContext };
  await Promise.all(
    queryList.map(async ({ definition, diffForIndex }, index) => {
      if (typeof diffForIndex !== "undefined") return;
      const modifiedQuery = await invokeHooks(
        "before",
        { definition },
        hookCallerOptions
      );
      queryList[index].definition = modifiedQuery.definition;
    })
  );
  await Promise.all(
    queryList.map(async ({ definition }, index) => {
      const modifiedQuery = await invokeHooks(
        "during",
        { definition },
        hookCallerOptions
      );
      queryList[index].result = modifiedQuery.result;
    })
  );
  const queriesWithoutResults = queryList.map((query, index) => ({ ...query, index })).filter((query) => query.result === EMPTY);
  if (queriesWithoutResults.length === 0)
    return queryList.map(({ result }) => result);
  const resultsFromDatabase = await runQueries(
    queriesWithoutResults.map(({ definition }) => definition),
    options
  );
  for (let index = 0; index < resultsFromDatabase.length; index++) {
    const query = queriesWithoutResults[index];
    const result = resultsFromDatabase[index];
    queryList[query.index].result = result;
  }
  for (let index = 0; index < queryList.length; index++) {
    const query = queryList[index];
    const queryType = Object.keys(query.definition)[0];
    if (queryType !== "create" && queryType !== "set" && queryType !== "drop") {
      continue;
    }
    const diffMatch = queryList.find((item) => item.diffForIndex === index);
    let resultBefore = diffMatch ? diffMatch.result : EMPTY;
    let resultAfter = query.result;
    if (queryType === "drop") {
      resultBefore = query.result;
      resultAfter = EMPTY;
    }
    const promise = invokeHooks(
      "after",
      { definition: query.definition, resultBefore, resultAfter },
      hookCallerOptions
    );
    const clearPromise = promise.then(
      () => {
      },
      (error) => Promise.reject(error)
    );
    if (waitUntil) waitUntil(clearPromise);
  }
  return queryList.filter((query) => typeof query.diffForIndex === "undefined").map(({ result }) => result);
};

// src/queries.ts
var runQueries = async (queries, options = {}) => {
  const hasWriteQuery = queries.some(
    (query) => ["create", "set", "drop"].includes(Object.keys(query)[0])
  );
  const hasCachingSupport = "cache" in new Request("https://ronin.co");
  const request = new Request("https://data.ronin.co/", {
    method: "POST",
    headers: {
      "Content-Type": "application/json",
      Authorization: `Bearer ${options.token}`
    },
    body: JSON.stringify({ queries }),
    // Disable cache if write queries are performed, as those must be
    // guaranteed to reach RONIN.
    ...hasWriteQuery && hasCachingSupport ? { cache: "no-store" } : {},
    // Allow for passing custom `fetch` options (e.g. in Next.js).
    ...typeof options?.fetch === "object" ? options.fetch : {}
  });
  const fetcher = typeof options?.fetch === "function" ? options.fetch : fetch;
  const response = await fetcher(request);
  const { results } = await getResponseBody(response);
  const startFormatting = performance.now();
  for (let index = 0; index < results.length; index++) {
    const result = results[index];
    if ("error" in result && result.error) {
      const message = result.error.code === "BAD_REQUEST" ? "Invalid query provided." : result.error.message;
      const path = result.error.issues?.[0]?.path ? getDotNotatedPath(result.error.issues[0].path) : null;
      const query = path && typeof result.error.issues[0].path[1] === "number" ? queries[result.error.issues[0].path[1]] : null;
      const instruction = query && path ? getProperty(query, path.replace(/queries\[\d+\]\./, "")) : null;
      const details = result.error.issues?.[0] ? result.error.issues[0].message : null;
      throw new InvalidQueryError({
        message,
        query: query && path ? `${path.replace(/queries\[\d+\]\./, "")} = ${JSON.stringify(instruction)}` : null,
        path,
        details,
        code: result.error.code || null,
        fields: result.error.fields || null
      });
    }
    if ("amount" in result && typeof result.amount !== "undefined" && result.amount !== null) {
      results[index] = Number(result.amount);
      continue;
    }
    const dateFields = "schema" in result ? Object.entries(result.schema).filter(([, type]) => type === "date").map(([name]) => name) : [];
    if ("record" in result) {
      if (result.record === null) {
        results[index] = null;
        continue;
      }
      formatDateFields(result.record, dateFields);
      results[index] = result.record;
      continue;
    }
    if ("records" in result) {
      for (const record of result.records) {
        formatDateFields(record, dateFields);
      }
      if (typeof result.moreBefore !== "undefined")
        result.records.moreBefore = result.moreBefore;
      if (typeof result.moreAfter !== "undefined")
        result.records.moreAfter = result.moreAfter;
      results[index] = result.records;
    }
  }
  const endFormatting = performance.now();
  const VERBOSE_LOGGING = typeof process !== "undefined" && process?.env && process.env.__RENDER_DEBUG_LEVEL === "verbose" || typeof import.meta?.env !== "undefined" && import.meta.env.__RENDER_DEBUG_LEVEL === "verbose";
  if (VERBOSE_LOGGING) {
    console.log(`Formatting took ${endFormatting - startFormatting}ms`);
  }
  return results;
};
var runQueriesWithStorageAndHooks = async (queries, options = {}) => {
  const queriesPopulatedWithReferences = await processStorableObjects(
    queries,
    (objects) => {
      return uploadStorableObjects(objects, options);
    }
  );
  return runQueriesWithHooks(queriesPopulatedWithReferences, options);
};

// src/utils/constants.ts
var RONIN_SCHEMA_SYMBOLS = {
  QUERY: "__RONIN_QUERY",
  FIELD: "__RONIN_FIELD_",
  VALUE: "__RONIN_VALUE"
};

// src/syntax/utils.ts
var IN_BATCH_ASYNC;
var IN_BATCH_SYNC = false;
var getSyntaxProxy = (queryType, queryHandler) => {
  return new Proxy(
    {},
    {
      get(_target, prop) {
        function createProxy(path) {
          const proxyTargetFunction = () => {
          };
          proxyTargetFunction();
          return new Proxy(proxyTargetFunction, {
            apply(_target2, _thisArg, args) {
              let value = args[0];
              const options = args[1];
              if (typeof value === "function") {
                IN_BATCH_SYNC = true;
                value = { [RONIN_SCHEMA_SYMBOLS.QUERY]: value().query };
                IN_BATCH_SYNC = false;
              }
              const expanded = objectFromAccessor(
                path.join("."),
                typeof value === "undefined" ? {} : value
              );
              const query = { [queryType]: expanded };
              if (IN_BATCH_ASYNC?.getStore() || IN_BATCH_SYNC) {
                return { query, options };
              }
              return queryHandler(query, options);
            },
            get(_target2, nextProp) {
              return createProxy(path.concat([nextProp]));
            }
          });
        }
        return createProxy([prop]);
      }
    }
  );
};
var getBatchProxy = (operations, options = {}, queriesHandler) => {
  let queries = [];
  if (options.asyncContext) {
    IN_BATCH_ASYNC = options.asyncContext;
    queries = IN_BATCH_ASYNC.run(true, () => operations());
  } else {
    IN_BATCH_SYNC = true;
    queries = operations();
    IN_BATCH_SYNC = false;
  }
  return queriesHandler(queries);
};

export {
  InvalidQueryError,
  InvalidResponseError,
  processStorableObjects,
  mergeOptions,
  runQueriesWithStorageAndHooks,
  getSyntaxProxy,
  getBatchProxy
};
